-- AlertTest.applescript

--  Created by Shane Stanley.
--  <sstanley@myriad-com.com.au>.
--  'AppleScriptObjC Explored' <http://www.macosxautomation.com/applescript/apps/>

-- sample code using NSAlert+MyriadHelpers category

script AlertTest
	property parent : class "NSObject"
	property mainWindow : missing value -- connect in Interface Builder
	
	-- shows NSAlert as a sheet over window mainWindow	
	on showAlert_(sender)
		-- create an alert
		tell current application's NSAlert to set theAlert to makeAlert_buttons_text_("An alert", {"Cancel", "OK"}, "Further explanation")
		tell theAlert to showOver_calling_(mainWindow, {"alertDone:", me}) -- selector takes one argument, the name of the button
	end showAlert_
	
	on alertDone_(theResult) -- selector called when alert closed
		log theResult -- Name of button pressed
	end alertDone_
	
	
	-- shows modal alert
	on showAlertModal_(sender)
		-- create an alert
		tell current application's NSAlert to set theAlert to makeAlert_buttons_text_("An alert", {"Cancel", "OK"}, "Further explanation")
		tell theAlert to showModal() -- returns name of the button
		log result
	end showAlertModal_
	
	-- shows NSAlert as a sheet over window mainWindow giving up after 5 seconds
	on showExpiringAlert_(sender)
		-- create an alert
		tell current application's NSAlert to set theAlert to makeAlert_buttons_text_("An alert", {"Cancel", "OK"}, "Further explanation")
		tell theAlert to showOver_calling_wait_(mainWindow, {"expiringAlertDone:", me}, 5) -- selector takes one argument, the name of the button
	end showExpiringAlert_
	
	on expiringAlertDone_(theResult) -- selector called when alert closed
		log theResult -- Name of button pressed, or "Gave up"
	end expiringAlertDone_
	
	
	-- shows modal alert giving up after 5 seconds
	on showExpiringAlertModal_(sender)
		-- create an alert
		tell current application's NSAlert to set theAlert to makeAlert_buttons_text_("An alert", {"Cancel", "OK"}, "Further explanation")
		tell theAlert to showModalWait_(5) -- returns name of the button, or "Gave up"
		log result
	end showExpiringAlertModal_
	
end script
