//
//  NSPathControl+MyriadHelpers.m
//
//  Created by Shane Stanley on 29/11/2010. v1.0.0.
//  <sstanley@myriad-com.com.au>.
//  'AppleScriptObjC Explored' <http://www.macosxautomation.com/applescript/apps/>
//

#import "NSPathControl+MyriadHelpers.h"


@implementation NSPathControl (MyriadHelpers)

-(NSString *)HFSFilepath {
	NSURL *url = [self URL];
	CFStringRef pathHFS = CFURLCopyFileSystemPath((CFURLRef)url, kCFURLHFSPathStyle);
	if (!pathHFS) return nil;
		// add trailing colon if a folder but not package
	NSNumber *result;
	if ([url getResourceValue:&result forKey:NSURLIsDirectoryKey error:nil] == NO) {
		return NSMakeCollectable(pathHFS); 
	}
	if ([result isEqualTo:[NSNumber numberWithInt:1]]) {
		if ([url getResourceValue:&result forKey:NSURLIsPackageKey error:nil] == NO) {
			return NSMakeCollectable(pathHFS);
		}
		if ([result isEqualTo:[NSNumber numberWithInt:0]]) {
			return [NSMakeCollectable(pathHFS) stringByAppendingString:@":"]; // directory, not package
		} 
	}
	return NSMakeCollectable(pathHFS);
}	

-(void)setHFSFilepath:(NSString *)path {
	CFURLRef url = CFURLCreateWithFileSystemPath (NULL, (CFStringRef)path, 
												  kCFURLHFSPathStyle,
												  (Boolean)[path hasSuffix:@":"]);
	[self setURL:NSMakeCollectable(url)];
}

-(void)setPathAny:(id)item {
	if ([item isKindOfClass:[NSAppleEventDescriptor class]]) {
			// it's an AS class; 
		DescType type = [item descriptorType];
		if (type == typeAlias) {	//alias
			item = [item coerceToDescriptorType: typeFileURL];				
			NSString *string = [[NSString alloc] initWithData:[item data] encoding:NSUTF8StringEncoding];
			[self setURL:[NSURL URLWithString: string]];
			return;
		}
		if (type == typeFileURL) {	//file
			NSString *string = [[NSString alloc] initWithData:[item data] encoding:NSUTF8StringEncoding];
			[self setURL:[NSURL URLWithString: string]];	
			return;
		}
		return; // fall back; none of the above
	} else {
			// it's not an AS class; 
		if ([item isKindOfClass:[NSURL class]]) {
			[self setURL:item];
			return;
		}
		if ([item isKindOfClass:[NSString class]]) {
			if ([item hasPrefix:@"~"]) {
				[self setURL:[NSURL fileURLWithPath:[item stringByExpandingTildeInPath]]];
				return;
			}
			if ([item hasPrefix:@"/"]) {
				[self setURL:[NSURL fileURLWithPath:item]];
				return;
			}
			[self setHFSFilepath:item];	
		}
		return; // fall back; none of the above
	}	
}

@end
