//
//  ObjectWithFords.m
//
//  Created by Shane Stanley on 29/11/2010. v1.0.0.
//  <sstanley@myriad-com.com.au>.
//  'AppleScriptObjC Explored' <http://www.macosxautomation.com/applescript/apps/>


#import "ObjectWithFords.h"


@implementation ObjectWithFords

-(id)ford:(id)item {
	if ([item isKindOfClass:[NSAppleEventDescriptor class]]) {
			// it's an AS class; 
		switch ([item descriptorType]) {
			case typeLongDateTime:	//date
				return [self fordASDateToNSDate:item];
				break;
			case typeAlias:	//alias
				return [self fordAliasToNSURL:item];
				break;				
			case typeFileURL:	//file
				return [self fordFileToNSURL:item];
				break;				
			case typeData:	//data
				return [self fordASDataToNSData:item];
				break;
		}
		return item; // fall back; none of the above
	} else {
			// it's not an AS class; 
		if ([item isKindOfClass:[NSDate class]]) 
			return [self fordNSDateToASDate:item];
		if ([item isKindOfClass:[NSURL class]]) 
			return [self fordNSURLToHFSPath:item];
		if ([item isKindOfClass:[NSData class]]) 
			return [self fordNSDataToASData:item];
		return item; // text/lists/records will have been converted by bridge already
	}
}

-(id)fordDeep:(id)item {
	if ([item isKindOfClass:[NSArray class]]) 
		return [self fordListItems:item];
	if ([item isKindOfClass:[NSDictionary class]]) 
		return [self fordRecordValues:item];
		// fall back to ford:
	return [self ford:item];
}

-(NSMutableArray *)fordListItems:(NSArray *)list {
	NSMutableArray * result = [NSMutableArray array];
	for (id object in list) {
		[result addObject: [self fordDeep:object]];
	}
	return result;
}

-(NSMutableDictionary *)fordRecordValues:(NSDictionary *)record {
	NSArray *keys = [record allKeys];
	NSArray *values = [record allValues];
	int i, length;
	length = [keys count];
	NSMutableDictionary *result = [NSMutableDictionary dictionaryWithCapacity:length];
	for (i = 0;i < length;i++) {
		[result setObject:[self fordDeep:[values objectAtIndex:i]] 
				   forKey:[keys objectAtIndex:i]];
	}
	return result;
}

-(NSAppleEventDescriptor *)fordNSDataToASData:(NSData *)data {
    return [NSAppleEventDescriptor descriptorWithDescriptorType:typeData
														   data:data];        
}

-(NSData *)fordASDataToNSData:(NSAppleEventDescriptor *)data {
	return [data data];
}


-(NSURL *)fordAliasToNSURL:(NSAppleEventDescriptor *)alias {
	alias = [alias coerceToDescriptorType: typeFileURL];				
	NSString *string = [[NSString alloc] initWithData:[alias data] 
											 encoding:NSUTF8StringEncoding];
	return [NSURL URLWithString: string];	
}

-(NSURL *)fordFileToNSURL:(NSAppleEventDescriptor *)furl {
	NSString *string = [[NSString alloc] initWithData:[furl data] 
											 encoding:NSUTF8StringEncoding];
	return [NSURL URLWithString: string];	
}

-(NSString *)fordNSURLToHFSPath:(NSURL *)url {
	CFStringRef pathHFS = CFURLCopyFileSystemPath((CFURLRef)url, kCFURLHFSPathStyle);
	if (!pathHFS) return nil;
		// add trailing colon if a folder but not package
	NSNumber *result;
	if ([url getResourceValue:&result forKey:NSURLIsDirectoryKey error:nil] == NO) {
		return NSMakeCollectable(pathHFS); 
	}
	if ([result isEqualTo:[NSNumber numberWithInt:1]]) {
		if ([url getResourceValue:&result forKey:NSURLIsPackageKey error:nil] == NO) {
			return NSMakeCollectable(pathHFS);
		}
		if ([result isEqualTo:[NSNumber numberWithInt:0]]) {
			return [NSMakeCollectable(pathHFS) stringByAppendingString:@":"]; // directory, not package
		} 
	}
	return NSMakeCollectable(pathHFS);
}

-(NSString *)fordNSURLToHFSPathFull:(NSURL *)url {
	CFStringRef pathHFS = CFURLCopyFileSystemPath((CFURLRef)url, kCFURLHFSPathStyle);
	if (!pathHFS) return nil;
		// add trailing colon if a directory
	NSNumber *result;
	if ([url getResourceValue:&result forKey:NSURLIsDirectoryKey error:nil] == NO) {
		return NSMakeCollectable(pathHFS); 
	}
		if ([result isEqualTo:[NSNumber numberWithInt:1]]) {
			return [NSMakeCollectable(pathHFS) stringByAppendingString:@":"];
	}
	return NSMakeCollectable(pathHFS);
}

-(NSURL *)fordHFSPathToNSURL:(NSString *)path {
	CFURLRef url = CFURLCreateWithFileSystemPath (NULL, (CFStringRef)path, 
												  kCFURLHFSPathStyle,
												  (Boolean)[path hasSuffix:@":"]);
	return NSMakeCollectable(url);
}

-(NSURL *)fordAnyToURL:(id)item {
	if ([item isKindOfClass:[NSAppleEventDescriptor class]]) {
			// it's an AS class; 
		DescType type = [item descriptorType];
		if (type == typeAlias) {	//alias
			return [self fordAliasToNSURL:item];	
		}
		if (type == typeFileURL) {	//file
			return [self fordFileToNSURL:item];	
		}
		return nil; // fall back; none of the above
	} else {
			// it's not an AS class; 
		if ([item isKindOfClass:[NSURL class]]) {
			return item;
		}
		if ([item isKindOfClass:[NSString class]]) {
			if ([item hasPrefix:@"~"]) {
				return [NSURL fileURLWithPath:[item stringByExpandingTildeInPath]];
			}
			if ([item hasPrefix:@"/"]) {
				return [NSURL fileURLWithPath:item];
			}
			return [self fordHFSPathToNSURL:item];	
		}
		return nil; // fall back; none of the above
	}	
}

-(NSAppleEventDescriptor *)fordNSDateToASDate:(NSDate *)date {
    LongDateTime ldt;
    UCConvertCFAbsoluteTimeToLongDateTime(CFDateGetAbsoluteTime((CFDateRef)date), &ldt);
    return [NSAppleEventDescriptor descriptorWithDescriptorType:typeLongDateTime
                                                          bytes:&ldt length:sizeof(ldt)];        
}

-(NSDate *)fordASDateToNSDate:(NSAppleEventDescriptor *)date {
    CFAbsoluteTime absTime;
    LongDateTime ldt;
	NSData *data = [date data];
    [data getBytes:&ldt length:[data length]];
    UCConvertLongDateTimeToCFAbsoluteTime(ldt, &absTime);
    return (NSDate *)[NSDate dateWithTimeIntervalSinceReferenceDate:absTime];
}

-(NSAppleEventDescriptor *)fordDateString:(NSString *)string {
	int result, i, list[6];
		// y, m and d default to 1; could set other values as defaults
	for (i = 0;i <3;i++) list[i] = 1;
	for (i = 3;i <6;i++) list[i] = 0;
	i = 0;
	NSScanner *scanner = [NSScanner scannerWithString:string];
		// scan skipping all but decimals
	[scanner setCharactersToBeSkipped:[[NSCharacterSet decimalDigitCharacterSet] invertedSet]];
	while (([scanner scanInt:&result]) && (i <6)) {
		if ((i <3) && (result == 0)) {
			list[i++] = 1; // don't want 0 for y, m and d or it goes to previous
		} else {
			list[i++] = result;
		}
	}
	CFAbsoluteTime at;
	LongDateTime ldt;
	CFCalendarRef cal = CFCalendarCopyCurrent();
	CFCalendarComposeAbsoluteTime(cal, &at, "yMdHms", list[0], list[1], list[2], list[3], list[4], list[5]);
	UCConvertCFAbsoluteTimeToLongDateTime(at, &ldt);
    return [NSAppleEventDescriptor descriptorWithDescriptorType:typeLongDateTime
                                                          bytes:&ldt length:sizeof(ldt)];        
}

-(void)fordEvent {
	NSEvent *event;
	while (event = [NSApp nextEventMatchingMask:NSAnyEventMask untilDate:nil inMode:NSEventTrackingRunLoopMode dequeue:YES]) {
		[NSApp sendEvent: event];
	}		
}

-(id)theClass:(NSString *)name {
	return NSClassFromString(name);
}

@end
