-- ObjectWithFordsTests.applescript

--  Created by Shane Stanley.
--  <sstanley@myriad-com.com.au>.
--  'AppleScriptObjC Explored' <http://www.macosxautomation.com/applescript/apps/>

-- sample code using ObjectWithFords superclass

script ObjectWithFordsTests
	property parent : class "ObjectWithFords"
	property imageView : missing value
	
	on classWithClassStuff_(sender)
		-- simple use of ford: to make NS equivalents of text, lists and records
		log "## Starting classWithClassStuff_##"
		-- use to make NSString from text
		set theNSString to ford_("Some text") -- easier than calling stringWithString: on NSString
		log theNSString's uppercaseString() -- instance method works so it worked
		log "##"
		log "##"
		
		-- convert list to array
		set theList to {"M", "B", "Z", "Q", "A"}
		set theList to ford_(theList) -- easier than calling arrayWithArray: on NSArray
		log (theList's sortedArrayUsingSelector_("compare:")) -- instance method works so it worked
		log "##"
		log "##"
		
		-- convert record to dictionary
		set theRec to {firstName:"Fred", age:16}
		set theRec to ford_(theRec) -- easier than calling dictionaryWithDictionary: on NSDictionary
		log (theRec's allKeys()) -- instance method works so it worked
		
	end classWithClassStuff_
	
	on dateStuff_(sender)
		log "## Starting dateStuff_##"
		-- make date from string; doing it the standard AS way fails in AppleScriptObjC
		set theStrings to {"2001-12-01 00:00:00", "2001-14-01 00:00:00", "2001/10/03", "blah 2010-10//3, 11;59,30 whatever"} -- order matters, delimiter doesn't
		repeat with aString in theStrings
			set theDate to fordDateString_(aString)
			log aString & " becomes " & ((theDate as date) as text)
		end repeat
		log "##"
		log "##"
		
		-- convert AS date to NSDate
		set thisDate to current date
		log "Current date is " & (thisDate as text)
		set theNSDate to ford_(thisDate)
		log theNSDate -- NSDate
		log theNSDate's timeIntervalSince1970() -- instance method works so it worked
		
		-- convert NSDate to AS date
		set aDate to (ford_(theNSDate) as date) -- still need coercion on return
		log "AS date is " & (aDate as text) -- OK
		log "##"
		log "##"
	end dateStuff_
	
	on fileStuff_(sender)
		log "## Starting fileStuff_##"
		-- theClass_ method returns a class; easier (and clearer?) than "current application's NSString"
		tell theClass_("NSURL") to set theURL to fileURLWithPath_("/blah /blah/more/blah/")
		log theURL
		log theURL's |class|() -- NSURL
		log "##"
		log "##"
		
		-- use ford_ to convert NSURL to HFS path
		set hfsPath to ford_(theURL)
		log (hfsPath as text) -- hfs path
		log "##"
		log "##"
		
		-- convert from alias to NSURL
		set anAlias to "Macintosh HD:" as alias -- change to suit
		log ford_(anAlias) -- NSURL
		log "##"
		log "##"
		
		-- convert from file to NSURL
		set theFile to choose file name
		log "AS path is " & (theFile as text)
		log ford_(theFile) -- NSURL
	end fileStuff_
	
	on deepFordStuff_(sender)
		-- deepFord: does all that ford: does, but for lists/arrays and records/dictionaries it also converts any contained dates/files/aliases/data
		set someAlias to choose file with prompt "Choose any file"
		log "## Starting deepFordStuff_##"
		-- convert a list and its contents		
		set aList to {someAlias, current date}
		log ford_(aList) -- shows array of NSAppleEventDescriptors
		log fordDeep_(aList) -- shows array of NSURL and NSDate
		log "##"
		log "##"
		
		-- convert a record and its contents		
		set aRecord to {firstItem:someAlias, secondItem:(current date)}
		log ford_(aRecord) -- shows dictionary with NSAppleEventDescriptors
		log fordDeep_(aRecord) -- shows dictionary with NSURL and NSDate
		log "##"
		log "##"
		
		-- convert list of arrays and contents		
		set aList to {{firstItem:someAlias, secondItem:(current date)}, {firstItem:someAlias, secondItem:(current date)}, {firstItem:someAlias, secondItem:(current date)}, {firstItem:someAlias, secondItem:(current date)}, {firstItem:someAlias, secondItem:(current date)}}
		log ford_(aList) -- shows NSAppleEventDescriptors
		log fordDeep_(aList) -- shows NSURLs and NSDates
	end deepFordStuff_
	
	on dataFromiTunes_(sender)
		log "## Starting dataFromiTunes_##"
		-- convert data to NSData; gets data for album cover from iTunes
		## iTunes must be running and a track playing or paused
		tell application "iTunes"
			set myTrack to (current track)
			set theArts to artworks of myTrack
			if theArts = {} then
				--do nothing
				return
			else
				set theArts to item 1 of theArts
				set imageData to raw data of theArts -- get the data
			end if
		end tell
		
		set theData to ford_(imageData) -- convert it to NSData
		tell current application's NSImage to set theImage to alloc()'s initWithData_(theData) -- turn it into an NSImage
		tell imageView to setImage_(theImage) -- put it in the image well
	end dataFromiTunes_
	
end script
