-- OpenSavePanelTests.applescript

--  Created by Shane Stanley.
--  <sstanley@myriad-com.com.au>.
--  'AppleScriptObjC Explored' <http://www.macosxautomation.com/applescript/apps/>

-- sample code using open and save panel categories

script OpenSavePanelTests
	property parent : class "NSObject"
	property mainWindow : missing value -- connect in Interface Builder
	
	## Show open panel as sheet
	-- shows NSOpenPanel over window mainWindow	
	on showOpenPanel_(sender)
		tell current application's NSOpenPanel to set thePanel to makeOpenAt_types_files_multiples_prompt_title_(path to desktop, {"txt", "pdf"}, true, false, "Choose a text or PDF file:", "My Open Title")
		tell thePanel to showOver_calling_(mainWindow, {"openSheetDone:", me})
	end showOpenPanel_
	
	on openSheetDone_(theResult) -- called when panel closed
		if theResult = missing value then
			-- cancel button
			log "Cancel pressed"
		else
			log theResult -- path to file, or list of files if multiples is true
		end if
	end openSheetDone_
	
	-- show modal open panel
	on showOpenPanelModal_(sender)
		tell current application's NSOpenPanel to set thePanel to makeOpenAt_types_files_multiples_prompt_title_(path to desktop, {"txt", "pdf"}, true, false, "Choose a text or PDF file:", "My Open Title")
		tell thePanel to set thePath to showModal() -- returns HFS path, or list if multiples is true
		if thePath = missing value then
			log "Cancel pressed"
		else
			log thePath
		end if
	end showOpenPanelModal_
	
	## Show save panel as sheet
	-- shows NSSavePanel over window mainWindow	
	on showSavePanel_(sender)
		tell current application's NSSavePanel to set thePanel to makeSaveAt_types_name_prompt_title_(path to home folder, {"txt"}, "Some name", "Save the data:", "My Save Title")
		tell thePanel to showOver_calling_(mainWindow, {"saveSheetDone:", me})
	end showSavePanel_
	
	on saveSheetDone_(theResult) -- called when panel closed
		if theResult = missing value then
			-- cancel button
			log "Cancel pressed"
		else
			log theResult -- path to file
		end if
	end saveSheetDone_
	
	-- show modal save panel
	on showSavePanelModal_(sender)
		tell current application's NSSavePanel to set thePanel to makeSaveAt_types_name_prompt_title_(path to home folder, {"txt"}, "Some name", "Save the data:", "My Save Title")
		tell thePanel to set thePath to showModal() -- returns HFS path
		if thePath = missing value then
			log "Cancel pressed"
		else
			log thePath
		end if
	end showSavePanelModal_
	
end script
